import pandas as pd
import sys
import os
import fnmatch
from numpy import linalg as LA
import numpy as np

from IBK import Delphin6GeoFileManager
from IBK import Delphin6GeoFile
from IBK import Delphin6OutputFile
from IBK import IntegralDifferenceNorm

def compare_files(projectFile, sliceTimePoints):
	"""Reads files in reference_results subdirectory and compares
	them with output files generated from project file.

	Arguments:
	   projectFile - path to project file
	   sliceTimePoints - list with time points that are used for 3D data files
	"""

	(basename, extension) = os.path.splitext(projectFile)
	if len(basename) < 1:
		print "Invalid project file argument."
		return 1

	# compose path names from project file path
	resultsFolder =  basename + '/results'
	referenceFolder = os.path.dirname(projectFile) + "/reference_results"

	# glob all files in referenceFolder
	matches = []
	for root, dirnames, filenames in os.walk(referenceFolder):
		for filename in fnmatch.filter(filenames, '*.d6o'):
			matches.append(filename)
	matches.sort()

	# create object for GeometryFileManager
	geoFileManager = Delphin6GeoFileManager()

	# for each result file, read output file and generate L2 norm of differences
	for fn in matches:
		referenceFileName = os.path.join(referenceFolder, fn)
		resultFileName = os.path.join(resultsFolder, fn)

		# create objects and read output data
		dfReference = Delphin6OutputFile()
		if not dfReference.read(referenceFileName, geoFileManager):
			print 'Error reading reference data file {}'.format(referenceFileName)
			return 1

		dfResults = Delphin6OutputFile()
		if not dfResults.read(resultFileName, geoFileManager):
			print 'Error reading results data file {}'.format(resultFileName)
			return 1

		# create pandas data sets
		dsReference = dfReference.asDataFrame()
		dsResults = dfResults.asDataFrame()

		# skip 3D data sets
		if len(dsReference.keys()) > 1:
			# check all time points individually
			for timePoint in sliceTimePoints:
				cutReference = dsReference[timePoint:timePoint].transpose()
				cutResult = dsResults[timePoint:timePoint].transpose()
				# compute differences
				diffnorm = IntegralDifferenceNorm.integral_difference_norm_steps(dfResults.steps, cutResult[timePoint], cutReference[timePoint])

				print '{:20} : delta = {}'.format( "{}[{}]".format(fn, timePoint), diffnorm)
				if abs(diffnorm) > 0.1:
					print 'Difference too large.'
					return 1

		else:
			# compute differences
			diffnorm = IntegralDifferenceNorm.integral_difference_norm_timepoints(dfResults.timePoints, dsResults.iloc[0:,0].values, dsResults.iloc[0:,0].values)
			print '{:20} : delta = {}'.format(fn, diffnorm)
			if abs(diffnorm) > 0.1:
				print 'Difference too large.'
				return 1

	return 0



